/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <SDL/SDL.h>

#include "m6809.h"
#include "pia.h"
#include "sound.h"
#include "logging.h"

#define SOUND_SAMPLE_RATE 22050
#define SOUND_CHANNELS 0
#define SOUND_BUFFER_SETTING 0x00010009
#define SOUND_FRAGMENT_SIZE 512
#define SOUND_BUFFER_SIZE (SOUND_FRAGMENT_SIZE + (SOUND_FRAGMENT_SIZE / 2))
#define SOUND_CYCLES_PER_SAMPLE (M6809_CPU_RATE / SOUND_SAMPLE_RATE)
#define SOUND_CYCLES_PER_FRAGMENT ((M6809_CPU_RATE * SOUND_FRAGMENT_SIZE) / SOUND_SAMPLE_RATE)

extern int cycle;

static int sound_fd = -1;
static Uint8 buffer[SOUND_BUFFER_SIZE];
static Uint8 *wrptr;
static Uint8 lastsample;
static float start_cycle;
static int sound_enable;

void sound_init(void) {
	int fragment = SOUND_BUFFER_SETTING;
	int channels = SOUND_CHANNELS;
	int format = AFMT_U8;
	int rate = SOUND_SAMPLE_RATE;
	sound_fd = open("/dev/dsp", O_WRONLY);
	if (sound_fd == -1) {
		LOG_ERROR0("Couldn't open sound device!\n");
		return;
	}
	if (ioctl(sound_fd, SNDCTL_DSP_SETFRAGMENT, &fragment) == -1) {
		LOG_ERROR0("Couldn't set buffer size/fragmentation limit!\n");
		return;
	}
	if (ioctl(sound_fd, SNDCTL_DSP_STEREO, &channels) == -1) {
		LOG_ERROR0("Couldn't set channels!\n");
		return;
	}
	if (ioctl(sound_fd, SNDCTL_DSP_SETFMT, &format) == -1) {
		LOG_ERROR0("Couldn't set audio format.\n");
		return;
	}
	if (ioctl(sound_fd, SNDCTL_DSP_SPEED, &rate) == -1) {
		LOG_ERROR0("Couldn't set audio rate.\n");
		return;
	}
	LOG_DEBUG2("Set up sound device at %dHz, %d channels.\n", rate, channels+1);
	PIA_1A.wrfunc[0] = sound_write_byte;
	wrptr = buffer;
	start_cycle = 0.0;
	sound_enable = 0;
}

void sound_reset(void) {
}

void sound_write_byte(Uint8 byte) {
	Uint8 *new_wrptr = buffer + (int)((cycle - start_cycle)/SOUND_CYCLES_PER_SAMPLE);
	if (!(PIA_1B.control_register & 0x08))
		byte = lastsample; // 0x80;
	if (new_wrptr < buffer || new_wrptr >= buffer + SOUND_BUFFER_SIZE) {
		start_cycle = (float)cycle;
		wrptr = buffer;
		lastsample = byte;
		return;
	}
	memset(wrptr, lastsample, new_wrptr - wrptr);
	wrptr = new_wrptr;
	if (new_wrptr >= buffer + SOUND_FRAGMENT_SIZE) {
		write(sound_fd, buffer, SOUND_FRAGMENT_SIZE);
		/*if (write(sound_fd, buffer, SOUND_FRAGMENT_SIZE) == -1) {
			fprintf(stderr, ".");
		} */
		wrptr -= SOUND_FRAGMENT_SIZE;
		start_cycle -= SOUND_CYCLES_PER_FRAGMENT;
		memmove(buffer + SOUND_FRAGMENT_SIZE, buffer, SOUND_BUFFER_SIZE - SOUND_FRAGMENT_SIZE);
	}
	lastsample = byte;
}
