/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>

#include "m6809.h"
#include "sam.h"
#include "vdg.h"
#include "pia.h"
#include "keyboard.h"
#include "sound.h"
#include "logging.h"

int cycle = 0;

int main(int argc, char **argv) {
	printf("XRoar, (c) 2003 Ciaran Anscomb.\n");

	/* Initialise SDL */
	if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO) < 0) {
		LOG_ERROR1("Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);

	/* Initialise all devices */
	sam_init();
	/* pia_init() must be called before devices that connect to PIAs */
	pia_init();
	m6809_init();
	vdg_init();
	keyboard_init();
	sound_init();

	/* Reset everything */
	keyboard_reset();
	sam_reset();
	vdg_reset();
	m6809_reset();
	sound_reset();

	for (; ; cycle++) {
		if (cycle % 8 == 0)
			keyboard_poll();
		m6809_cycle();
		vdg_cycle();
		if (cycle % 10 == 0) {
			sound_write_byte(PIA_1A.output_register);
		}
	}

	return 0;
}
